@@ -63,7 +63,6 @@
 
 struct _CamelEwsStorePrivate {
 
-	gchar *host_url;
 	time_t last_refresh_time;
 	GMutex *get_finfo_lock;
 	EEwsConnection *cnc;
@@ -71,8 +70,7 @@ struct _CamelEwsStorePrivate {
 
 extern CamelServiceAuthType camel_ews_password_authtype; /*for the query_auth_types function*/
 static gboolean	ews_store_construct	(CamelService *service, CamelSession *session,
-			 		 CamelProvider *provider, CamelURL *url,
-		 			 GError **error);
+			 		 CamelProvider *provider, GError **error);
 
 static void camel_ews_store_initable_init (GInitableIface *interface);
 static GInitableIface *parent_initable_interface;
@@ -82,6 +80,48 @@ G_DEFINE_TYPE_WITH_CODE (
 	G_IMPLEMENT_INTERFACE (
 		G_TYPE_INITABLE, camel_ews_store_initable_init))
 
+static void
+ews_migrate_to_user_cache_dir (CamelService *service)
+{
+	const gchar *user_data_dir, *user_cache_dir;
+
+	g_return_if_fail (service != NULL);
+	g_return_if_fail (CAMEL_IS_SERVICE (service));
+
+	user_data_dir = camel_service_get_user_data_dir (service);
+	user_cache_dir = camel_service_get_user_cache_dir (service);
+
+	g_return_if_fail (user_data_dir != NULL);
+	g_return_if_fail (user_cache_dir != NULL);
+
+	/* migrate only if the source directory exists and the destination doesn't */
+	if (g_file_test (user_data_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR) &&
+	    !g_file_test (user_cache_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
+		gchar *parent_dir;
+
+		parent_dir = g_path_get_dirname (user_cache_dir);
+		g_mkdir_with_parents (parent_dir, S_IRWXU);
+		g_free (parent_dir);
+
+		if (g_rename (user_data_dir, user_cache_dir) == -1) {
+			g_debug ("%s: Failed to migrate '%s' to '%s': %s", G_STRFUNC, user_data_dir, user_cache_dir, g_strerror (errno));
+		} else {
+			gchar *old_summary_file = g_build_filename (user_cache_dir, "folder-tree-v2", NULL);
+
+			if (old_summary_file && g_file_test (old_summary_file, G_FILE_TEST_EXISTS)) {
+				gchar *new_summary_file = g_build_filename (user_cache_dir, "folder-tree", NULL);
+
+				if (new_summary_file && g_rename (old_summary_file, new_summary_file) == -1)
+					g_debug ("%s: Failed to migrate '%s' to '%s': %s", G_STRFUNC, old_summary_file, new_summary_file, g_strerror (errno));
+
+				g_free (new_summary_file);
+			}
+
+			g_free (old_summary_file);
+		}
+	}
+}
+
 static gboolean
 ews_store_initable_init		(GInitable *initable,
 				 GCancellable *cancellable,
@@ -89,18 +129,21 @@ ews_store_initable_init		(GInitable *initable,
 {
 	CamelService *service;
 	CamelSession *session;
-	CamelURL *url;
+	CamelStore *store;
 	gboolean ret;
-	
+
+	store = CAMEL_STORE (initable);
 	service = CAMEL_SERVICE (initable);
-	url = camel_service_get_camel_url (service);
 	session = camel_service_get_session (service);
 
+	store->flags |= CAMEL_STORE_USE_CACHE_DIR;
+	ews_migrate_to_user_cache_dir (service);
+
 	/* Chain up to parent interface's init() method. */
 	if (!parent_initable_interface->init (initable, cancellable, error))
 		return FALSE;
 
-	ret = ews_store_construct (service, session, NULL, url, error);
+	ret = ews_store_construct (service, session, NULL, error);
 
 	/* Add transport here ? */
 
@@ -117,8 +160,7 @@ camel_ews_store_initable_init (GInitableIface *interface)
 
 static gboolean
 ews_store_construct	(CamelService *service, CamelSession *session,
-			 CamelProvider *provider, CamelURL *url,
-			 GError **error)
+			 CamelProvider *provider, GError **error)
 {
 	CamelEwsStore *ews_store;
 	CamelEwsStorePrivate *priv;
@@ -133,7 +175,7 @@ ews_store_construct	(CamelService *service, CamelSession *session,
 	((CamelStore *)ews_store)->flags &= ~(CAMEL_STORE_VTRASH|CAMEL_STORE_VJUNK);
 
 	/*storage path*/
-	session_storage_path = g_strdup (camel_service_get_user_data_dir (service));
+	session_storage_path = g_strdup (camel_service_get_user_cache_dir (service));
 	if (!session_storage_path) {
 		g_set_error (
 			error, CAMEL_STORE_ERROR,
@@ -143,19 +185,10 @@ ews_store_construct	(CamelService *service, CamelSession *session,
 	}
 	ews_store->storage_path = session_storage_path;
 
-	priv->host_url = g_strdup (camel_url_get_param (url, "hosturl"));
-	if (!priv->host_url) {
-		g_set_error (
-			error, CAMEL_STORE_ERROR,
-			CAMEL_STORE_ERROR_INVALID,
-			_("EWS service has no host URL"));
-		return FALSE;
-	}
-
 	/* Note. update account-listener plugin if filename is changed here, as it would remove the summary
 	   by forming the path itself */
 	g_mkdir_with_parents (ews_store->storage_path, 0700);
-	summary_file = g_build_filename (ews_store->storage_path, "folder-tree-v2", NULL);
+	summary_file = g_build_filename (ews_store->storage_path, "folder-tree", NULL);
 	ews_store->summary = camel_ews_store_summary_new (summary_file);
 	camel_ews_store_summary_load (ews_store->summary, NULL);
 
@@ -200,15 +233,21 @@ ews_store_authenticate	(EEwsConnection *cnc,
 			 SoupMessage *msg, SoupAuth *auth,
 			 gboolean retrying, gpointer data)
 {
+	CamelNetworkSettings *network_settings;
+	CamelSettings *settings;
 	CamelService *service = data;
-	CamelURL *url;
 	const gchar *password;
+	const gchar *user;
 
-	url = camel_service_get_camel_url (service);
 	password = camel_service_get_password (service);
+	settings = camel_service_get_settings (service);
+
+	network_settings = CAMEL_NETWORK_SETTINGS (settings);
+	user = camel_network_settings_get_user (network_settings);
+
 	g_return_if_fail (password != NULL);
 
-	e_ews_connection_authenticate (cnc, auth, url->user, password, NULL);
+	e_ews_connection_authenticate (cnc, auth, user, password, NULL);
 }
 
 static gboolean
@@ -216,14 +255,24 @@ ews_connect_sync (CamelService *service, GCancellable *cancellable, GError **err
 {
 	CamelEwsStore *ews_store;
 	CamelEwsStorePrivate *priv;
+	CamelEwsSettings *ews_settings;
+	CamelNetworkSettings *network_settings;
+	CamelSettings *settings;
 	CamelSession *session;
-	CamelURL *url;
+	const gchar *hosturl;
+	const gchar *user;
 	gboolean success;
 
 	ews_store = (CamelEwsStore *) service;
 	priv = ews_store->priv;
-	url = camel_service_get_camel_url (service);
 	session = camel_service_get_session (service);
+	settings = camel_service_get_settings (service);
+
+	network_settings = CAMEL_NETWORK_SETTINGS (settings);
+	user = camel_network_settings_get_user (network_settings);
+
+	ews_settings = CAMEL_EWS_SETTINGS (settings);
+	hosturl = camel_ews_settings_get_hosturl (ews_settings);
 
 	if (camel_service_get_connection_status (service) == CAMEL_SERVICE_DISCONNECTED)
 		return FALSE;
@@ -235,7 +284,7 @@ ews_connect_sync (CamelService *service, GCancellable *cancellable, GError **err
 		return TRUE;
 	}
 
-	priv->cnc = e_ews_connection_new (priv->host_url, url->user, NULL,
+	priv->cnc = e_ews_connection_new (hosturl, user, NULL,
 					  G_CALLBACK (ews_store_authenticate), service,
 					  error);
 
@@ -824,15 +873,22 @@ ews_rename_folder_sync	(CamelStore *store,
 gchar *
 ews_get_name (CamelService *service, gboolean brief)
 {
-	CamelURL *url;
+	CamelNetworkSettings *network_settings;
+	CamelSettings *settings;
+	const gchar *host;
+	const gchar *user;
 
-	url = camel_service_get_camel_url (service);
+	settings = camel_service_get_settings (service);
+
+	network_settings = CAMEL_NETWORK_SETTINGS (settings);
+	host = camel_network_settings_get_host (network_settings);
+	user = camel_network_settings_get_user (network_settings);
 
 	if (brief)
-		return g_strdup_printf(_("Exchange server %s"), url->host);
+		return g_strdup_printf(_("Exchange server %s"), host);
 	else
 		return g_strdup_printf(_("Exchange service for %s on %s"),
-				       url->user, url->host);
+				       user, host);
 }
 
 EEwsConnection *
@@ -906,7 +962,6 @@ ews_store_finalize (GObject *object)
 	ews_store = CAMEL_EWS_STORE (object);
 
 	g_free (ews_store->storage_path);
-	g_free (ews_store->priv->host_url);
 	g_mutex_free (ews_store->priv->get_finfo_lock);
 
 	/* Chain up to parent's finalize() method. */
